import serial
import serial.tools.list_ports

from ..base import LedControlDevice

import logging
log = logging.getLogger(__name__)

ARDUINO_SOFTWARE_VERSION_STRING = "Arduino Nano CPD 1"
class LEDD1B(LedControlDevice):
    """
    Control a Thorlabs LEDD1B LED driver using an Arduino Nano.
    The arduino must have the correct software loaded on it.
    (See `arduino-thorlabs-ledd1b`project directory.)

    Note: This currently has COM4 hardcoded
    """
    def __init__(self, port="COM4"):
        super().__init__()
        self.arduino = serial.Serial(port=port, baudrate=9600, timeout=0.1)
        # flush input
        while self.arduino.read(1):
            pass

    def __del__(self):
        self.off()
        self.arduino.close()
    
    def check_has_correct_software(self, no_error=False):
        """
        Run the identify command and raise an Exception
        if the Arduino does not reply with the expected output.
        """
        self._write('i')
        lines = self.read()
        if len(lines) < 1 or not lines[-1].startswith(bytes(ARDUINO_SOFTWARE_VERSION_STRING, "utf-8")):
            log.error(f"Arduino did not return the expected output - does it have the correct software loaded?\nExpected: '{ARDUINO_SOFTWARE_VERSION_STRING}'\nReceived:{lines}")
            if not no_error:
                raise ConnectionError("Arduino did not return the expected output - does it have the correct software loaded?")

    def test_connection(self) -> None:
        try:
            self._write('i')
            lines = self.read()
        except Exception as e:
            log.error(f"Failed to query 'i':", e)
            raise ConnectionError

    def _write(self, val):
        self.arduino.write(bytes(val, 'utf-8'))
        self.arduino.flush()
        
    def read(self):
        data = self.arduino.readlines()
        return data
       
    def on(self):
        self._write("1")
    def off(self):
        self._write("0")
    def set_level(self, level:int):
        if level == 0: self.off()
        elif level == 100: self.on()
        else:
            raise ValueError(f"LEDD1B Led controller can only set 0% or 100%")

    def __str__(self):
        return "Thorlabs LEDD1B"

    @staticmethod
    def enumerate_devices() -> list[str]:
        devs = []
        coms = serial.tools.list_ports.comports()
        # Require a device with no product field and empty serial number
        # Change these conditions to support different devices
        for com in coms:
            if com.product is not None: continue
            if com.serial_number: continue
            if com.manufacturer not in ["wch.cn"]: continue
            devs.append(com.device)
        return devs

    @staticmethod
    def connect_device(name: str):
        return LEDD1B(port=name)

   
if __name__ == '__main__':
    led = LEDD1B()